//===========================================================================
//  $Name: arts++-1-1-a5 $
//  $Id: ArtsFileUtil.hh,v 1.7 1999/09/03 09:35:34 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSFILEUTIL_HH_
#define _ARTSFILEUTIL_HH_

#include <string>
#include <vector>
#include <fstream.h>

#include "ArtsProtocolTableAggregatorMap.hh"
#include "ArtsInterfaceMatrixAggregatorMap.hh"
#include "ArtsNextHopTableAggregatorMap.hh"
#include "ArtsAsMatrixAggregatorMap.hh"
#include "ArtsNetMatrixAggregatorMap.hh"
#include "ArtsPortMatrixAggregatorMap.hh"
#include "ArtsTosTableAggregatorMap.hh"
#include "Arts.hh"
#include "ArtsSelectionSet.hh"
#include "ArtsIfIndexSelectionSet.hh"
#include "ArtsObjectTypeSelectionSet.hh"
#include "ArtsTimeIntervalSelectionSet.hh"

//----------------------------------------------------------------------------
//                            class ArtsFileUtil 
//----------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
class ArtsFileUtil
{
public:
  //--------------------------------------------------------------------------
  //                             ArtsFileUtil() 
  //..........................................................................
  //  constructor
  //--------------------------------------------------------------------------
  ArtsFileUtil();

  //--------------------------------------------------------------------------
  //             ArtsFileUtil(const ArtsFileUtil & artsFileUtil) 
  //..........................................................................
  //  copy constructor
  //--------------------------------------------------------------------------
  ArtsFileUtil(const ArtsFileUtil & artsFileUtil);

  //--------------------------------------------------------------------------
  //                             ~ArtsFileUtil() 
  //..........................................................................
  //  destructor
  //--------------------------------------------------------------------------
  ~ArtsFileUtil();
  
  //--------------------------------------------------------------------------
  //     bool Aggregate(const string & outFileName,
  //                    const vector<string> & inFileNames,
  //                    float hours, bool overwrite, bool quiet) const
  //..........................................................................
  //  Aggregates data in inFileNames and places the results in a file
  //  named by outFileName.  Hours is the minimum number of hours that
  //  should be in each aggregate object produced.  A value of 0.0 for
  //  hours will create an aggregate that covers the entire time period
  //  seen in the input files.  If overwrite is true, we overwrite the
  //  output file, else we append to it.  If quiet is false we print a
  //  '.' character for each object we read as input and a '+' character
  //  for each object we write as output.  If _portChooser has port choices
  //  in it, we will generate selected port tables instead of port
  //  matrices when we have port matrices in the input.
  //--------------------------------------------------------------------------
  bool Aggregate(const string & outFileName,
                 const vector<string> & inFileNames,
                 float hours, bool overwrite, bool quiet) const;

  //--------------------------------------------------------------------------
  //         bool AggregateProtocolTables(const string & outFilename, 
  //                                      const vector<string> & inFilenames, 
  //                                float hours, bool overwrite, 
  //                                bool quiet) const 
  //..........................................................................
  //  Aggregates protocol table objects in inFileNames and places the
  //  results in a file named by outFileName.  Hours is the minimum
  //  number of hours that should be in each aggregate object produced.
  //  A value of 0.0 for hours will create an aggregate that covers the
  //  entire time period seen in the input files.  If overwrite is true,
  //  we overwrite the output file, else we append to it.  If quiet is
  //  false we print a '.' character for each object we read as input
  //  and a '+' character for each object we write as output.  Returns
  //  false if unable to open output file, else returns true.
  //--------------------------------------------------------------------------
  bool AggregateProtocolTables(const string & outFilename,
                               const vector<string> & inFilenames,
                               float hours, bool overwrite,
                               bool quiet) const;
  
  //--------------------------------------------------------------------------
  //        bool AggregateTosTables(const string & outFilename, 
  //                                const vector<string> & inFilenames, 
  //                                float hours, bool overwrite, 
  //                                bool quiet) const 
  //..........................................................................
  //  Aggregates TOS table objects in inFileNames and places the results
  //  in a file named by outFileName.  Hours is the minimum number of
  //  hours that should be in each aggregate object produced.  A value
  //  of 0.0 for hours will create an aggregate that covers the entire
  //  time period seen in the input files.  If overwrite is true, we
  //  overwrite the output file, else we append to it.  If quiet is
  //  false we print a '.' character for each object we read as input
  //  and a '+' character for each object we write as output.  Returns
  //  false if unable to open output file, else returns true.
  //--------------------------------------------------------------------------
  bool AggregateTosTables(const string & outFilename,
                          const vector<string> & inFilenames,
                          float hours, bool overwrite,
                          bool quiet) const;

  //--------------------------------------------------------------------------
  //   bool AggregateInterfaceMatrices(const string & outFilename, 
  //                                   const vector<string> & inFilenames, 
  //                                   float hours, bool overwrite, 
  //                                   bool quiet) const 
  //..........................................................................
  //  Aggregates interface matrix objects in inFileNames and places the
  //  results in a file named by outFileName.  Hours is the minimum
  //  number of hours that should be in each aggregate object produced.
  //  A value of 0.0 for hours will create an aggregate that covers the
  //  entire time period seen in the input files.  If overwrite is true,
  //  we overwrite the output file, else we append to it.  If quiet is
  //  false we print a '.' character for each object we read as input
  //  and a '+' character for each object we write as output.  Returns
  //  false if unable to open output file, else returns true.
  //--------------------------------------------------------------------------
  bool AggregateInterfaceMatrices(const string & outFilename,
                                  const vector<string> & inFilenames,
                                  float hours, bool overwrite,
                                  bool quiet) const;

  //--------------------------------------------------------------------------
  //     bool AggregateNextHopTables(const string & outFilename, 
  //                                 const vector<string> & inFilenames, 
  //                                 float hours, bool overwrite, 
  //                                 bool quiet) const 
  //..........................................................................
  //  Aggregates IP nexthop table objects in inFileNames and places the
  //  results in a file named by outFileName.  Hours is the minimum
  //  number of hours that should be in each aggregate object produced.
  //  A value of 0.0 for hours will create an aggregate that covers the
  //  entire time period seen in the input files.  If overwrite is true,
  //  we overwrite the output file, else we append to it.  If quiet is
  //  false we print a '.' character for each object we read as input
  //  and a '+' character for each object we write as output.  Returns
  //  false if unable to open output file, else returns true.
  //--------------------------------------------------------------------------
  bool AggregateNextHopTables(const string & outFilename,
                              const vector<string> & inFilenames,
                              float hours, bool overwrite,
                              bool quiet) const;

  //--------------------------------------------------------------------------
  //           bool AggregateAsMatrices(const string & outFilename, 
  //                                    const vector<string> & inFilenames, 
  //                            float hours, bool overwrite, 
  //                            bool quiet) const 
  //..........................................................................
  //  Aggregates AS matrix objects in inFileNames and places the results
  //  in a file named by outFileName.  Hours is the minimum number of
  //  hours that should be in each aggregate object produced.  A value
  //  of 0.0 for hours will create an aggregate that covers the entire
  //  time period seen in the input files.  If overwrite is true, we
  //  overwrite the output file, else we append to it.  If quiet is
  //  false we print a '.' character for each object we read as input
  //  and a '+' character for each object we write as output.  Returns
  //  false if unable to open output file, else returns true.
  //--------------------------------------------------------------------------
  bool AggregateAsMatrices(const string & outFilename,
                           const vector<string> & inFilenames,
                           float hours, bool overwrite,
                           bool quiet) const;

  //--------------------------------------------------------------------------
  //       bool AggregateNetMatrices(const string & outFilename, 
  //                                 const vector<string> & inFilenames, 
  //                                 float hours, bool overwrite, 
  //                                 bool quiet) const 
  //..........................................................................
  //  Aggregates net matrix objects in inFileNames and places the
  //  results in a file named by outFileName.  Hours is the minimum
  //  number of hours that should be in each aggregate object produced.
  //  A value of 0.0 for hours will create an aggregate that covers the
  //  entire time period seen in the input files.  If overwrite is true,
  //  we overwrite the output file, else we append to it.  If quiet is
  //  false we print a '.' character for each object we read as input
  //  and a '+' character for each object we write as output.  Returns
  //  false if unable to open output file, else returns true.
  //--------------------------------------------------------------------------
  bool AggregateNetMatrices(const string & outFilename,
                            const vector<string> & inFilenames,
                            float hours, bool overwrite,
                            bool quiet) const;

  //--------------------------------------------------------------------------
  //    bool AggregatePortMatrices(const string & outFilename, 
  //                               const vector<string> & inFilenames, 
  //                               float hours, bool overwrite,
  //                               bool quiet) const 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool AggregatePortMatrices(const string & outFilename,
                             const vector<string> & inFilenames,
                             float hours, bool overwrite,
                             bool quiet) const;

  void PortChooser(const char *portRangeExpression);
  void IfIndexSelectionSet(ArtsIfIndexSelectionSet & ifIndexSet);
  void IfIndexSelectionSet(const char *ifIndexSelectionExpression);
  
  void IfIndexCombineSet(ArtsIfIndexSelectionSet & ifIndexCombineSet);
  void IfIndexCombineSet(const char *ifIndexSelectionExpression);
  
  void ObjectTypeSelectionSet(ArtsObjectTypeSelectionSet & objectTypeSet);
  void ObjectTypeSelectionSet(const char *objTypeSelectionExpression);
  
  void
  TimeIntervalSelectionSet(ArtsTimeIntervalSelectionSet & timeIntervalSet);
  void TimeIntervalSelectionSet(const char *startTimeExpression,
                                const char *endTimeExpression);
  
  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                     static uint32_t NumObjects()
    //........................................................................
    //
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG
  
private:

  ArtsPortChooser               _portChooser;
  ArtsIfIndexSelectionSet       _ifIndexSelectionSet;
  ArtsIfIndexSelectionSet       _ifIndexCombineSet;
  ArtsObjectTypeSelectionSet    _objectTypeSelectionSet;
  ArtsTimeIntervalSelectionSet  _timeIntervalSelectionSet;
  
  void FinishProtocolTableAgg(ArtsProtocolTableAggregatorMap & protoAggMap,
                              ofstream & out, bool quiet) const;
  void FinishTosTableAgg(ArtsTosTableAggregatorMap & tosAggMap,
                         ofstream & out, bool quiet) const;
  void FinishInterfaceMatrixAgg(ArtsInterfaceMatrixAggregatorMap & intfmAggMap,
                                ofstream & out, bool quiet) const;
  void FinishNextHopTableAgg(ArtsNextHopTableAggregatorMap & nextHopAggMap,
                             ofstream & out, bool quiet) const;
  void FinishAsMatrixAgg(ArtsAsMatrixAggregatorMap & asmAggMap,
                         ofstream & out, bool quiet) const;
  void FinishNetMatrixAgg(ArtsNetMatrixAggregatorMap & netmAggMap,
                          ofstream & out, bool quiet) const;
  void FinishPortMatrixAgg(ArtsPortMatrixAggregatorMap & portmAggMap,
                           ofstream & out, bool quiet) const;
  void AggregateProtocolTableData(ArtsProtocolTableAggregatorMap & protoAggMap,
                                  const Arts & arts, ofstream & out,
                                  float hours, bool quiet) const;
  void AggregateTosTableData(ArtsTosTableAggregatorMap & tosAggMap,
                             const Arts & arts, ofstream & out,
                             float hours, bool quiet) const;
  void AggregateInterfaceMatrixData(ArtsInterfaceMatrixAggregatorMap & intfmAggMap,
                                    const Arts & arts, ofstream & out,
                                    float hours, bool quiet) const;
  void AggregateNextHopTable(ArtsNextHopTableAggregatorMap & nextHopAggMap,
                             const Arts & arts, ofstream & out, float hours,
                             bool quiet) const;
  void AggregateAsMatrixData(ArtsAsMatrixAggregatorMap & asmAggMap,
                             const Arts & arts, ofstream & out, float hours,
                             bool quiet) const;
  void AggregateNetMatrixData(ArtsNetMatrixAggregatorMap & netmAggMap,
                              const Arts & arts, ofstream & out, float hours,
                              bool quiet) const;
  void AggregatePortMatrixData(ArtsPortMatrixAggregatorMap & portmAggMap,
                               const Arts & arts, ofstream & out,
                               float hours, bool quiet) const;

  static uint32_t  _numObjects;

};

#endif  // _ARTSFILEUTIL_HH_
